package main

import (
	"encoding/base64"
	"encoding/binary"
	"encoding/json"
	"io/ioutil"
	"log"
	"math/rand"
	"os"

	"srs.epita.fr/fic-server/libfic"
)

type wantChoices struct {
	FlagId int64 `json:"id"`
}

func treatWantChoices(pathname string, team fic.Team) {
	// Generate a unique identifier to follow the request in logs
	bid := make([]byte, 5)
	binary.LittleEndian.PutUint32(bid, rand.Uint32())
	id := "[" + base64.StdEncoding.EncodeToString(bid) + "]"
	log.Println(id, "New wantChoices receive", pathname)

	var ask wantChoices

	if cnt_raw, err := ioutil.ReadFile(pathname); err != nil {
		log.Printf("%s [ERR] %s\n", id, err)
	} else if err = json.Unmarshal(cnt_raw, &ask); err != nil {
		log.Printf("%s [ERR] %s\n", id, err)
	} else if ask.FlagId == 0 {
		log.Printf("%s [WRN] Invalid content in wantChoices file:  %s\n", id, pathname)
		os.Remove(pathname)
	} else if flag, err := fic.GetFlagKey(ask.FlagId); err != nil {
		log.Printf("%s [ERR] %s\n", id, err)
	} else if !team.CanSeeFlag(flag) {
		log.Printf("%s [!!!] The team asks to display choices whereas it doesn't have access to the flag\n", id)
	} else if exercice, err := flag.GetExercice(); err != nil {
		log.Printf("%s [ERR] Unable to retrieve the flag's underlying exercice: %s\n", id, err)
	} else if !team.HasAccess(exercice) {
		log.Printf("%s [!!!] The team asks to display choices whereas it doesn't have access to the exercice\n", id)
	} else if err = team.DisplayChoices(flag); err != nil {
		log.Printf("%s [ERR] %s\n", id, err)
	} else {
		genTeamQueue <- &team
		if err = os.Remove(pathname); err != nil {
			log.Printf("%s [ERR] %s\n", id, err)
		}
	}
}
